@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: cf.c 335 2023-09-26 20:56:33Z leres $ (XSE)
+ * @(#) $Id: cf.c 341 2023-10-06 21:02:44Z leres $ (XSE)
  *
  * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2015, 2017, 2018, 2019, 2020, 2022, 2023
  *	Craig Leres
@@ -39,20 +39,112 @@
 #define DEF_MAXCOL	5
 
 /* Forwards */
+static void addtz(struct cf *, struct tzlist *, const char *);
 static char *dstrmerge(char *, char *);
 static void freelist(struct list *);
+static void freetz(struct cf *);
 static int keysort(const void *, const void *);
 static struct list *makelist(char *, const char *);
 static int readcf(const char *, struct cf *, int);
 static void fixpath(char **);
 
+static void
+addtz(struct cf *cf, struct tzlist *tl, const char *at)
+{
+	struct tzlist *tl2, *tl3;
+
+	tl2 = (struct tzlist *)calloc(1, sizeof(*tl2));
+	if (tl2 == NULL) {
+		fprintf(stderr, "%s: %s addtz: calloc: %s\n",
+		    prog, at, strerror(errno));
+		exit(1);
+	}
+	*tl2 = *tl;
+	tl = tl2;
+	tl->t_tz = strdup(tl->t_tz);
+	if (tl->t_tz == NULL) {
+		fprintf(stderr, "%s: %s addtz: calloc: %s\n",
+		    prog, at, strerror(errno));
+		exit(1);
+	}
+
+	/* Insert in order */
+	if (cf->c_tzlist == NULL) {
+		cf->c_tzlist = tl;
+		return;
+	}
+
+	tl2 = cf->c_tzlist;
+	tl3 = NULL;
+	// XXX complain about duplicates or use last one seen?
+	while (tl2->t_start < tl->t_start) {
+		if (tl2->t_next == NULL) {
+			tl2->t_next = tl;
+			tl = NULL;
+			break;
+		}
+		tl3 = tl2;
+		tl2 = tl2->t_next;
+	}
+	if (tl != NULL) {
+		if (tl3 == NULL) {
+			tl->t_next = cf->c_tzlist;
+			cf->c_tzlist = tl;
+		} else {
+			tl->t_next = tl3->t_next;
+			tl3->t_next = tl;
+		}
+	}
+}
+
+static void
+freetz(struct cf *cf)
+{
+	struct tzlist *tl, *tl2;
+
+	tl = cf->c_tzlist;
+	if (tl == NULL)
+		return;
+	tl2 = tl;
+	for (;;) {
+		free(tl2->t_tz);
+		tl2->t_tz = NULL;
+		free(tl2);
+		tl2 = NULL;
+		tl = tl->t_next;
+		if (tl == NULL)
+			break;
+		tl2 = tl;
+	}
+}
+
+const char *
+gettz(struct cf *cf, time_t t)
+{
+	struct tzlist *tl, *tl2;
+
+	tl2 = NULL;
+	for (tl = cf->c_tzlist; tl != NULL; tl = tl->t_next) {
+		if (tl->t_start != 0 && t < tl->t_start)
+			break;
+		tl2 = tl;
+	}
+	if (tl2 == NULL)
+		return (NULL);
+
+	return (tl2->t_tz);
+}
+
 /* Open, read and parse one config file */
 static int
 readcf(const char *fn, struct cf *cf, int mustexist)
 {
 	int n, d, errs, *ip;
-	char *cp, *cp2, *prefixend, **pp;
+	size_t len;
+	u_long uv;
+	char *cp, *cp2, *ep, *prefixend, **pp;
 	struct timespec ts;
+	struct tzlist *tl, ttzlist;
 	FILE *f, *f2;
 	struct section *sp, *sp2;
 	struct list **lpp;
@@ -129,8 +221,7 @@ readcf(const char *fn, struct cf *cf, int mustexist)
 		    (pp = &cf->c_invisiblepat,
 			strcasecmp(cp2, "invisiblepat") == 0) ||
 		    (pp = &cf->c_naildir, strcasecmp(cp2, "naildir") == 0) ||
-		    (pp = &cf->c_target, strcasecmp(cp2, "target") == 0) ||
-		    (pp = &cf->c_tz, strcasecmp(cp2, "tz") == 0)) {
+		    (pp = &cf->c_target, strcasecmp(cp2, "target") == 0)) {
 			cp3 = cp2;
 			if (*cp++ != '"') {
 				fprintf(stderr,
@@ -470,6 +561,54 @@ readcf(const char *fn, struct cf *cf, int mustexist)
 			continue;
 		}
 
+		/* Timezones */
+		if (strcasecmp(cp2, "tz") == 0) {
+			tl = &ttzlist;
+			memset(tl, 0, sizeof(*tl));
+			if (*cp++ != '"') {
+				fprintf(stderr,
+				    "%s: %s missing leading double quote\n",
+				    prog, at);
+				++errors;
+				continue;
+			}
+			cp2 = strchr(cp, '"');
+			if (cp2 == NULL) {
+				fprintf(stderr,
+				    "%s: %s missing trailing double quote\n",
+				    prog, at);
+				++errors;
+				continue;
+			}
+			len = cp2 - cp;
+			if (len > sizeof(tname))
+				len = sizeof(tname);
+			strncpy(tname, cp, len);
+			tname[len] = '\0';
+			tl->t_tz = tname;
+
+			cp = cp2 + 1;
+			while (isspace(*cp) && *cp != '\n')
+				++cp;
+
+			if (*cp != '\0') {
+				uv = strtoul(cp, &ep, 10);
+				if (*ep != '\0') {
+					fprintf(stderr,
+					    "%s: %s Invalid tz timestamp\n",
+					    prog, at);
+					++errors;
+					continue;
+				}
+				tl->t_start = uv;
+			}
+
+			/* Add this timezone*/
+			addtz(cf, tl, at);
+
+			continue;
+		}
+
 		/* Bad commands */
 		fprintf(stderr, "%s: %s unknown command \"%s\"\n",
 		    prog, at, cp2);
@@ -601,8 +740,6 @@ freecf(struct cf *cf)
 		free(cf->c_favicon);
 	if (cf->c_target != NULL)
 		free(cf->c_target);
-	if (cf->c_tz != NULL)
-		free(cf->c_tz);
 
 	/* Multi-line options */
 	if (cf->c_body != NULL)
@@ -639,6 +776,10 @@ freecf(struct cf *cf)
 			free(sp->s_html);
 		free(sp);
 	}
+
+	/* Timezone */
+	freetz(cf);
+
 	free(cf);
 }
 
