@@ -1,4 +1,4 @@
-/* @(#) $Id: cf.h 335 2023-09-26 20:56:33Z leres $ (XSE) */
+/* @(#) $Id: cf.h 341 2023-10-06 21:02:44Z leres $ (XSE) */
 #ifndef cf_h
 #define cf_h
 
@@ -41,6 +41,12 @@ struct list {
 	int l_flags;
 };
 
+struct tzlist {
+	struct tzlist *t_next;
+	time_t t_start;
+	char *t_tz;
+};
+
 /* Flags */
 #define L_REFERENCED	0x01		/* referenced */
 
@@ -69,7 +75,6 @@ struct cf {
 	char *c_invisiblepat;
 	char *c_naildir;
 	char *c_target;
-	char *c_tz;
 
 	/* Multi-line options */
 	char *c_body;
@@ -88,8 +93,12 @@ struct cf {
 
 	/* Section */
 	struct section *c_section;
+
+	/* timezone foo */
+	struct tzlist *c_tzlist;
 };
 
 struct cf *parsecf(const char *);
 void freecf(struct cf *);
+const char *gettz(struct cf *, time_t);
 #endif
