@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: webindex.c 340 2023-09-28 03:02:07Z leres $ (XSE)
+ * @(#) $Id: webindex.c 341 2023-10-06 21:02:44Z leres $ (XSE)
  *
  * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015, 2016, 2017, 2018, 2019, 2022, 2023
  *	Craig Leres
@@ -121,6 +121,8 @@ extern char *optarg;
 extern int optind, opterr, optopt;
 
 /* Locals */
+static const char tz[] = "TZ";
+static const char *currenttz;
 static const char magic[] = "This file produced by webindex; do not edit";
 static const char tnlowsuffix[] = "_t.jpg";
 static const char tnuppersuffix[] = "_t.JPG";
@@ -292,7 +294,7 @@ process(const char *dname)
 	const char *p, *indexfn;
 	const char *table, *tr, *tail;
 	char *cp, *section, *prefixend;
-	int i, row, col, force, intable, savedtz;
+	int i, row, col, force, intable;
 	struct timespec file_tim, ind_tim;
 	struct webfile *wp, *webfiles;
 	struct cf *cf;
@@ -306,7 +308,6 @@ process(const char *dname)
 	DIRENTRY *dp;
 	struct stat sbuf;
 	char buf[128];
-	static char tz[] = "TZ";
 
 	if (verbose > 1)
 		fprintf(stderr, "%s: processing %s%s%s\n",
@@ -330,12 +331,23 @@ process(const char *dname)
 	tr = "<tr>";
 	tail = "\n";
 	otz = NULL;
-	savedtz = 0;
+	currenttz = NULL;
 
 	/* Save our current working directory */
 	if (!pushd(dname, &prefixend, &d))
 		goto bail;
 
+	/* Save the timezone */
+	otz = getenv(tz);
+	if (otz != NULL) {
+		otz = strdup(otz);
+		if (otz == NULL) {
+			fprintf(stderr, "%s: process: strdup: %s\n",
+			    prog, strerror(errno));
+			exit(1);
+		}
+	}
+
 	/* Read configuration info */
 	cf = parsecf(configfn);
 	if (cf->c_indexfn != NULL)
@@ -354,21 +366,6 @@ process(const char *dname)
 		cf->c_tnsuffix = tnuppersuffix;
 	cf->c_tnsuffixlen = strlen(cf->c_tnsuffix);
 
-	/* Set timezone */
-	if (cf->c_tz != NULL) {
-		++savedtz;
-		otz = getenv(tz);
-		if (otz != NULL) {
-			otz = strdup(otz);
-			if (otz == NULL) {
-				fprintf(stderr, "%s: process: strdup: %s\n",
-				    prog, strerror(errno));
-				exit(1);
-			}
-		}
-		setenv(tz, cf->c_tz, 1);
-	}
-
 	/* Open the directory for scanning */
 	dirp = opendir(".");
 	if (dirp == NULL) {
@@ -719,17 +716,15 @@ bail:
 	(void)popd(prefixend, d);
 
 	/* Restore timezone */
-	if (savedtz) {
-		if (otz != NULL) {
-			setenv(tz, otz, 1);
-			free(otz);
-		} else {
+	if (otz != NULL) {
+		setenv(tz, otz, 1);
+		free(otz);
+	} else {
 #ifdef HAVE_UNSETENV
-			unsetenv(tz);
+		unsetenv(tz);
 #else
-			setenv(tz, "", 1);
+		setenv(tz, "", 1);
 #endif
-		}
 	}
 
 	/* Close the index file */
@@ -756,7 +751,7 @@ processfile(FILE *f, struct webfile *wp, struct cf *cf,
     const char *table, const char *tr, const char *indexfn)
 {
 	char *cp, *cp2, *name;
-	const char *p, *icon, *alt, *linkpostfix;
+	const char *p, *icon, *alt, *linkpostfix, *newtz;
 	struct webfile *wp2;
 	struct list *lp;
 	int i, j, k, w, h;
@@ -927,6 +922,12 @@ processfile(FILE *f, struct webfile *wp, struct cf *cf,
 		}
 
 		if (wp->w_timestamp > 0) {
+			newtz = gettz(cf, wp->w_timestamp);
+			if (currenttz != newtz) {
+				currenttz = newtz;
+				setenv(tz, currenttz, 1);
+			}
+
 			/* XXX timezone correction hack */
 			t = wp->w_timestamp;
 			if (cf->c_tzhours != 0)
